###########
#
#  Copyright 2002, Tom Hoffman.
# 
#  This file is part of Tuttle.
#
#  Tuttle is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  Tuttle is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Tuttle (see COPYING.txt); if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##########

from OFS.Folder import Folder
from Globals import DTMLFile
from StudentsFolder import StudentsFolder
from TeachersFolder import TeachersFolder
from zLOG import LOG, INFO
from Day import Day
import string

def manage_addSchoolInfo(self,id,title,\
district,superintendent,REQUEST):
	"Method for adding a Tuttle site"
	newSchool = SchoolInfo(id,title,district,superintendent)
	# add a folder for students
	studFolder=StudentsFolder('student', title + ' Student Records')
	newSchool._setObject('student',studFolder)
	# add a folder for teachers
	teachFolder=TeachersFolder('teacher', title + ' Teacher Files')
	newSchool._setObject('teacher',teachFolder)
	self._setObject(id, newSchool)
	
	# add a ZCatalog and a bunch of index info

	try:
		self.manage_addProduct['ZCatalog'].manage_addZCatalog('Catalog','ZCatalog for Tuttle')
	
	except:
		LOG("Tuttle", INFO, "There's already a ZCatalog called 'Catalog.'")
	
	else:
		# Student and teacher properties
		self.Catalog.manage_addIndex('FirstName', 'FieldIndex')
		self.Catalog.manage_addIndex('MiddleName', 'FieldIndex')
		self.Catalog.manage_addIndex('LastName', 'FieldIndex')
		self.Catalog.manage_addIndex('PreferredName', 'FieldIndex')
		self.Catalog.manage_addIndex('Team', 'FieldIndex')
		self.Catalog.manage_addIndex('Gender', 'FieldIndex')
		# Course properties
		self.Catalog.manage_addIndex('CourseName', 'FieldIndex')
		self.Catalog.manage_addIndex('Subject', 'FieldIndex')
		self.Catalog.manage_addIndex('Level','FieldIndex')
		self.Catalog.manage_addIndex('Description', 'TextIndex')
		self.Catalog.manage_addIndex('Roster', 'TextIndex')
		
	
	return self.manage_main(self, REQUEST)

# web interface methods
manage_addSchoolInfoForm = DTMLFile("dtml/addSchoolInfoForm", globals())

class SchoolInfo(Folder):
	"""
	The SchoolInfo Class
	This class is the heart of a Tuttle Site.
	It contains the default site configuration options,
	as well as the basic "API" for returning student
	and teacher data
	"""
	
	meta_type = "Tuttle School Site"
	
	################################
	#                              #
	#  Site Configuration Options  #
	#                              # 
	################################
	
	# Question:
	# Do these need to be under __init__ for SchoolInfo?

	# A list of the grades or levels in the school
	# e.g., in traditional (high) schools, GradeLevelList would be ['9','10','11','12']
	GradeLevelList=['9', '10','11','12']
	
	# If students and teachers are organized into teams, academies, etc.
	TeamList=['red','blue','yellow','black'] # these teams include teachers and students
	TeacherTeamList = TeamList + ['No Team'] # additional "teams" for teachers only
	
	# These need to be updated in a few years...
	# Graduation Years
	GradYearList=["2003","2004","2005","2006"]

	# These are all the subjects taught in the school.
	SubjectList=['English','Social Studies','Mathematics','Science','ESL',\
	'Health and Physical Education','Spanish','French','Art','Music']

	# Periods
	# Each period with a unique combination of classes across the school
	# needs a unique identifier.
	PeriodList=['A','B','C','D','E','F']
	
	Genders=('Male','Female')

	def __init__ (self, id, title,\
	district="Unknown School District", \
	superintendent="Unknown"):
		"To do: add more of the attributes in the SIF model."
		self.id=id
		self.title=title # maps to SIF.SchoolInfo.SchoolName
		self.district=district
		self.superintendent=superintendent
		self.Days={}
		self.Modules=[]
	
	# set up management tabs
	manage_options=Folder.manage_options[:3] + (
        {'label':'Configure Schedule','action':'manage_Schedule'},
        ) + Folder.manage_options[3:]
				

	
	# declare properties
	_properties=(
	{'id':'title', 'type':'string'},
	{'id':'district', 'type':'string'},
	{'id':'superintendent','type':'string'},
	{'id':'GradeLevelList','type':'lines'},
	{'id':'TeamList','type':'lines'},
	{'id':'TeacherTeamList','type':'lines'},
	{'id':'SubjectList','type':'lines'},
	{'id':'PeriodList','type':'lines'},
	{'id':'GradYearList','type':'lines'}
	)

	# Class level functions and (web) methods 
	index_html=DTMLFile("dtml/indexSchoolInfo", globals())
	manage_Schedule=DTMLFile("dtml/manageSchedule",globals())

	def editSchoolInfo(self, title, district, superintendent,REQUEST):
		"Edit School Properties"
		
		self.title=title
		self.district=district
		self.superintendent=superintendent
		return self.index_html(self, REQUEST)
		
	##########################
	#                        #
	#  Setting the schedule  #
	#                        #
	##########################
	 
	# creating a data structure flexible enough to handle most
	# schedules a school is likely to create is a small nightmare
	# Here's what we've got
	#
	# A period is a unique schoolwide configuration of classes.
	# PeriodList is a list of the arbitrary names of all periods.
	#
	# A module is a defined block of time.
	# Modules is a list of dictionaries.
	# The keys are 'id', 'start' and 'end'.
	# The values for 'id' should be integers, starting
	# at 0 and going chronologically through the day.
	#
	# A day is a unique sequence of periods, 
	# currently assumed to be within a calendar day.
	# Days is a dictionary.
	# The keys are the arbitrary names of each day;
	# the values are tuples of the sequence of periods in that day.
	
	def addDay(self, day, REQUEST=None):
		"Adds an entry for a day to the Day dictionary"
		thisDay = Day(day)
		self.Days[day] = thisDay
		self.__changed__(1)
		if REQUEST is not None:
			return self.manage_Schedule(self, REQUEST)

	def delDay(self, delID, REQUEST=None):
		"Delete some days"
		del self.Days[delID]
		self.__changed__(1)
		if REQUEST is not None:
			return self.manage_Schedule(self.REQUEST)
		
	def addModules(self, count, REQUEST=None):
		"Create a set of modules"
		if self.Modules == []:
			for x in range(count):
				thisModule={'id':x}
				self.Modules.append(thisModule)
			self.__changed__(1)
		if REQUEST is not None:
			return self.manage_Schedule(self.REQUEST)

	def editModule(self, mod, start, end, REQUEST=None):
		"Edits  module start and end times"
		for x in self.Modules:
			if x['id']==mod:
				x['start']=start
				x['end']=end
		self.__changed__(1)
		if REQUEST is not None:
			return self.manage_Schedule(self.REQUEST)
			
	def delMods(self, REQUEST=None):
		"Just clears all the modules"
		self.Modules=[]
		self.__changed__(1)
		if REQUEST is not None:
			return self.manage_Schedule(self.REQUEST)
			
	def setDayPeriods(self, day, sched, REQUEST=None):
		"Sets a tuple of periods as a value in the Days dictionary"
		self.Days[day]=sched
		self.__changed__(1)
		if REQUEST is not None:
			return self.manage_Schedule(self.REQUEST)
			
	def delDayPeriods(self, day, REQUEST=None):
		"Clears a day's periods"
		self.Days[day]=()
		self.__changed__(1)
		if REQUEST is not None:
			return self.manage_Schedule(self.REQUEST)
		

	######################
	#                    #
	# Tuttle API methods #
	#                    #
	######################

	def getInfo (self, role, userid, Method):
		"""This method allows you to make an arbitrary call on a
		student or teacher object
		role is 'student' or 'teacher'"""
		call=""
		CodeString = "call = self." + role +  "." + userid + "." + Method
		exec CodeString
		return call

	# these methods cover various formats for people's names

	def getFullName (self, role, userid):
		"Returns 'First Middle Last'"
		first = self.getInfo (role, userid, 'FirstName')
		middle = self.getInfo (role, userid, 'MiddleName')
		last = self.getInfo (role, userid, 'LastName')
		if middle:
			fullname=string.join([first,middle], " ")
		else:
			fullname=first
		fullname=string.join([fullname,last], " ")
		return fullname

	def getFullerName (self, role, userid):
		"Returns 'Prefix First Middle Last Suffix'"
		fullername = self.getFullName (role, userid)
		fullername = self.addPrefixSuffix (role, userid, fullername)
		return fullername

	def addPrefixSuffix (self, role, userid, name):
		"Adds prefix and suffix"
		prefix=""
		if role != 'student':
			prefix = self.getInfo (role, userid, 'Prefix')
		suffix = self.getInfo (role, userid, 'Suffix')
		if prefix :
			name=string.join([prefix, name], " ")
		if suffix:
			name=string.join([name, suffix], " ")
		return name

	def getFullestName (self, role, userid):
		"""Returns name in form 
		'Prefix FirstName MiddleName (PreferredName) LastName Suffix'"""
		fullestname = self.getFullName(role, userid)
		pref = self.getInfo (role, userid, 'PreferredName')
		if pref:		
			pref = string.join(['(', pref, ')'],"")
			pref = [pref]
			fullestname=string.split(fullestname, ' ',)
			fullestname=fullestname[:-1] + pref + fullestname[-1:]
			fullestname=string.join(fullestname, " ")
		fullestname = self.addPrefixSuffix(role, userid, fullestname)
		return fullestname

	def getTeamMembers (self, role, team="no team"):
		'Returns a list of ids for the members of a team, or all members'
		call = []
		result = []
		codestring = "call = self." + role + ".objectIds(\'" + role + "\')"
		exec codestring
		for name in call:
			if team == "no team" or team == self.getInfo(role, name, 'Team'):
				result.append (name)
		return result
				
